//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Oct 13 11:34:57 GMT-0800 1997
// Last Modified: Tue Feb 10 21:05:19 GMT-0800 1998
// Filename:      .../sig/code/control/Timer/Timer.h
// Syntax:        C++ 
// $Smake:        cc -g -c %b.cc && rm -f %b.o
//

#ifndef _TIMER_H_INCLUDED
#define _TIMER_H_INCLUDED


#ifdef VISUAL
   #include <wtypes.h>
   typedef LONGLONG int64bits;
#else
   typedef long long int int64bits;
#endif


class Timer {
   public:
                       Timer              (void);
                       Timer              (long aSpeed);
                       Timer              (Timer& aTimer);
                      ~Timer              ();

      int64bits        clockCycles        (void) const;
      double           expired            (void) const;
      long             getCpuSpeed        (void) const;
      double           getFactor          (void) const;
      double           getPeriod          (void) const;
      long             getTicksPerSecond  (void) const;
      long             getTime            (void) const;
      double           getTimeInSeconds   (void) const;
      long             getTimeInTicks     (void) const;
      long             measureCpuSpeed    (void) const;
      void             reset              (void);
      void             setCpuSpeed        (long aSpeed);
      void             setPeriod          (double aPeriod);
      void             setTempo           (double beatsPerMinute);
      void             setTicksPerSecond  (long aTickRate);
      void             start              (void);
      void             sync               (Timer& aTimer);
      void             update             (void);
      void             update             (int periodCount);

   protected:
      static int64bits globalOffset;
      int64bits        offset;          
      static long      cpuSpeed;         
      long             ticksPerSecond;    
      double           period;


};

   
#endif  _TIMER_H_INCLUDED



