//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed Feb 11 23:19:44 GMT-0800 1998
// Last Modified: Wed Feb 11 23:19:51 GMT-0800 1998
// Filename:      ...sig/code/control/improv/synthImprov.h
// Syntax:        C++
//
// Description: Environment for Synthesizer I/O.
//

#ifndef SYNTHIMPROV_H_INCLUDED
#define SYNTHIMPROV_H_INCLUDED

#include "improv.h"
#include <iostream.h>
#include <conio.h>          /* for kbhit() function */


// function declarations
int  checkKeyboard(void);
int  chooseSynthInputDevice(void);
int  chooseSynthOutputDevice(void);
void finishup_automatic(void);
void initialization_automatic(void);
void print_commands(void);
void print_aux_commands(void);
void octavekeyboard(int key);
void charsynth(int aKey);


// user defined functions:
void initialization(void);
void description(void);
void keyboardchar(int key);
void mainloopalgorithms(void);
void finishup(void);


////////////////////
// global variables

Synthesizer synth;    // MIDI I/O for a MIDI synthesizer
Timer mainTimer;      // Timer counting in milliseconds
long t_time = 0;      // global time (in milliseconds) which remains
                      // fixed for one pass through mainloopalgorithms()


///////////////////////////////////////////////////////////////////////////

int main(void) {
   t_time = mainTimer.getTime(); 
   initialization_automatic();
   print_commands();
   initialization();                  // user defined behavior

   while (checkKeyboard() != 'Q') {
      synth.processIncomingMessages();
      t_time = mainTimer.getTime(); 
      mainloopalgorithms();           // user defined behavior
   }

   finishup();                        // user defined behavior
   finishup_automatic();

   return 0;
}


///////////////////////////////////////////////////////////////////////////


//////////////////////////////
//
// checkKeyboard -- check to see if a key has been pressed on the keyboard.
//    If so, then interpret the keypress as a command and return the
//    keyboard character when done.  If the character is not used by
//    the main environment, then give the character to the user's function:
//    keyboardchar().
//
//    Commands below are listed in ASCII order.  Should not use here the
//    characters: a-z, 0-9, or: ,./;'[]\=-`, control characters,
//    or alt characters or the ESC key since these keys are
//    guarenteed to be available to the user for their own use.
//

int checkKeyboard(void) {
   if (!kbhit()) return -1;

   char command;
   command = getch();

   switch (command) {
   // case ' ': break;
   // case '!': break;
   // case '\"': break;
   // case '#': break;
   // case '$': break;
   // case '%': break;
   // case '&': break;
   // case '(': break;
   // case ')': break;
   // case '*': break;
   // case '\'': break;
   // case '-': break;
   // case '.': break;
   // case '/': break;
   // case '0': break;
   // case '1': break;
   // case '2': break;
   // case '3': break;
   // case '3': break;
   // case '4': break;
   // case '5': break;
   // case '6': break;
   // case '7': break;
   // case '8': break;
   // case '9': break;
   // case ':': break;
   // case ';': break;
   // case '<': break;
   // case '=': break;
   // case '>': break;
      case '?':                         // print main command menu
         print_commands();
         break;
   // case '@': break;
   // case 'A': break;
   // case 'B': break;
      case 'C':                         // change CPU speed variable
         {
         int aSpeed;
         cout << "Current speed is listed as: " 
              << (double)mainTimer.getCpuSpeed()/1000000.0 << " MHz.";
         cout << endl << "Enter the CPU speed of the computer in MHz: ";
         cin  >> aSpeed;
         mainTimer.setCpuSpeed(aSpeed * 1000000);
         }
         break;
      case 'D':                         // print description
         description(); // user defined function
         break;
   // case 'E': break;
   // case 'F': break;
   // case 'G': break; 
   // case 'H': break;
      case 'I':                         // set synthesizer input
         cout << "synthesizer MIDI input is currently set to port: " 
              << synth.getInputPort() << endl;
         synth.setInputPort(chooseSynthInputDevice());
         synth.openInput();
         cout << "synthesizer MIDI input is now set to port: "
              << synth.getInputPort() << endl;
         break;
   // case 'J': break;
   // case 'K': break;
   // case 'L': break;
      case 'M':                         // display more commands
         // this command displays less important commands
         print_aux_commands();
         break;
   // case 'N': break;
      case 'O':                         // set MIDI out device for synth
         cout << "synthesizer MIDI output is currently set to port: "
              << synth.getOutputPort() << endl;
         synth.setOutputPort(chooseSynthOutputDevice());
         synth.openOutput();
         cout << "synthesizer MIDI output is now set to port: "
              << synth.getOutputPort() << endl;
         break;
   // case 'P': break;
      case 'Q':                         // quit the program
         // don't do anything here, the calling function (main)
         // will handle this command.
         break;
   // case 'R': break;
      case 'S':                         // send note-offs to the synth
         cout << "Silencing synthesizer ...";
         synth.silence();
         cout << " done." << endl;
         break;
   // case 'T': break;
   // case 'U': break;
   // case 'V': break;
   // case 'W': break;
      case 'X':                         // toggle MIDI output trace
         synth.toggleOutputTrace();
         if (synth.getOutputTrace()) {
            cout << "MIDI output trace is ON." << endl;
         } else {
            cout << "MIDI output trace is OFF." << endl;
         }
         break;
      case 'Y':                         // toggle MIDI input trace
         synth.toggleInputTrace();
         if (synth.getInputTrace()) {
            cout << "MIDI input trace is ON." << endl;
         } else {
            cout << "MIDI input trace is OFF." << endl;
         }
         break;
   // case 'Z': break;
   // case '[': break;
   // case '\': break;
   // case ']': break;
   // case '^': break;
   // case '_': break;
   // case '`': break;
   // case 'a': break;
   // case 'b': break;
   // case 'c': break;
   // case 'd': break;
   // case 'e': break;
   // case 'f': break;
   // case 'g': break;
   // case 'h': break;
   // case 'i': break;
   // case 'j': break;
   // case 'k': break;
   // case 'l': break;
   // case 'm': break;
   // case 'n': break;
   // case 'o': break;
   // case 'p': break;
   // case 'q': break;
   // case 's': break;
   // case 't': break;
   // case 'u': break;
   // case 'v': break;
   // case 'w': break;
   // case 'x': break;
   // case 'y': break;
   // case 'z': break;
   // case '{': break;
   // case '|': break;
   // case '}': break;
   // case '~': break;
      default:                          // unknown command
         // can't identify command, so it must be a user command.
         // give the character to the user's keyboard function:
         keyboardchar(command);
   } // end of switch (command)

   return command;
}
   


//////////////////////////////
//
// chooseSynthInputDevice -- have the user select the
//    MIDI input device to communicate with the synthesizer
//

int chooseSynthInputDevice(void) {
   synth.pause();
   int deviceCount = synth.MidiInPort::getNumPorts();

   if (deviceCount <= 1) {
      synth.unpause();
      return 0;
   }

   int choice = -100;

   cout << "Choose one of the following MIDI input ports"
        << " for Synthesizer communication" << endl;
   displayInputDevices();

   while (choice < 0 || choice >= deviceCount) {
      cout << "Type port number choice here: ";
      cin  >> choice;
   }

   synth.unpause();
   return choice;
}



//////////////////////////////
//
// chooseSynthOutputDevice -- have the user select the 
//    MIDI output device to communicate with the synthesizer.
//

int chooseSynthOutputDevice(void) {
   synth.pause();
   int deviceCount = synth.MidiOutPort::getNumPorts();

   if (deviceCount <= 1) {
      synth.unpause();
      return 0;
   }

   int choice = -100;

   cout << "Choose one of the following MIDI output ports "
        << "for synthesizer communication:" << endl;
   displayOutputDevices();

   while (choice < -1 || choice >= deviceCount) {
      cout << "Type port number choice here: ";
      cin  >> choice;
   }

   synth.unpause();
   return choice;
}



//////////////////////////////
//
// finishup_automatic -- place automatic configuration 
//    commands here.
//

void finishup_automatic(void) {
   cout << endl;
}

     

//////////////////////////////
//
// initialization_automatic -- place automatic initialization
//    commands here.
//

void initialization_automatic(void) {
   // choose the MIDI out port for synthesizer
   synth.setOutputPort(chooseSynthOutputDevice());
   synth.openOutput();

   // choose the MIDI in port for synthesizer
   synth.setInputPort(chooseSynthInputDevice());
   synth.openInput();

   // enable MIDI input from the synthesizer (in case is starts paused)
   synth.unpause();
}



//////////////////////////////
//
// print_commands -- print keyboard character commands
//

void print_commands(void) {
   cout << endl;
   printboxtop();
   printcenterline(
"UPPERCASE COMMANDS :: SynthImprov Interface :: Music 14q :: Winter 1998");
   printintermediateline();
   printstringline(
"   D = print a description of this program given by the programmer         ");
   printstringline(
"   ? = print this command list         M = print more commands             ");
   printstringline(
"   Q = quit the program                S = silence synthesizer             ");
   printstringline(
"   X = toggle MIDI out trace           Y = toggle MIDI in trace           ");
   printstringline(
"   I = set MIDI in port for synth      O = set MIDI out port for synth    ");
   printboxbottom();
}


//////////////////////////////
//
// print_aux_commands -- display additional commands not in main menu.
//

void print_aux_commands(void) {
   cout << endl;
   printboxtop();
   printcenterline("Synth Improv Interface version 1/4/97 by");
   printcenterline("Craig Stuart Sapp  <craig@ccrma.stanford.edu>");
   printintermediateline();
   printstringline(
"   C = set CPU speed (used for timer)                                     ");
   printboxbottom();
}



//////////////////////////////
//
// octavekeyboard -- plays notes on bottom two rows of keys on
//	computer keyboard.  Octave transpositions can be made by
//	typing the octave numbers 0 through 9.
//

void octavekeyboard(int key, int octave) {
   static MidiMessage noteMessage;
   static int attack = 0;
   static int note = 0;

   synth.play(CH_1, note, 0);

   noteMessage.time = mainTimer.getTime();
   noteMessage.command() = 0x90;
   noteMessage.p1() = note;
   noteMessage.p2() = 0;
   synth.insert(noteMessage);

   switch (key) {
      case 'z': note = 12 * octave + 0;   break;   // C
      case 's': note = 12 * octave + 1;   break;   // C#
      case 'x': note = 12 * octave + 2;   break;   // D
      case 'd': note = 12 * octave + 3;   break;   // D#
      case 'c': note = 12 * octave + 4;   break;   // E
      case 'v': note = 12 * octave + 5;   break;   // F
      case 'g': note = 12 * octave + 6;   break;   // F#
      case 'b': note = 12 * octave + 7;   break;   // G
      case 'h': note = 12 * octave + 8;   break;   // G#
      case 'n': note = 12 * octave + 9;   break;   // A
      case 'j': note = 12 * octave + 10;  break;   // A#
      case 'm': note = 12 * octave + 11;  break;   // B
      case ',': note = 12 * octave + 12;  break;   // C
      default: return;
   }
   if (note < 0)  note = 0;
   else if (note > 127)  note = 127;

   attack = rand()%48 + 80; 
   synth.play(CH_1, note, attack);

   noteMessage.time = mainTimer.getTime();
   noteMessage.command() = 0x90;
   noteMessage.p1() = note;
   noteMessage.p2() = attack;
   synth.insert(noteMessage);
}



//////////////////////////////
//
// charsynth -- plays the synthesizer with lowercase characters
//    on the computer's keyboard (bottom two rows).
// z=C4, x=D4, c=E4, v=F4, b=G4, n=A4, m=B4, ","=C5, etc.
//

void charsynth(int aKey) {
   static MidiMessage noteMessage;
   static int attack = 0;
   static int note = 0;

   synth.play(CH_1, note, 0);

   noteMessage.time = mainTimer.getTime();
   noteMessage.command() = 0x90;
   noteMessage.p1() = note;
   noteMessage.p2() = 0;
   synth.insert(noteMessage);

   switch (aKey) {
      case 'z':  note=C4;    break;
      case 'x':  note=D4;    break;
      case 'c':  note=E4;    break;
      case 'v':  note=F4;    break;
      case 'b':  note=F4;    break;
      case 'n':  note=A4;    break;
      case 'm':  note=B4;    break;
      case ',':  note=C5;    break;
      case '.':  note=D5;    break;
      case '/':  note=E5;    break;
      case 'a':  note=B3;    break;
      case 's':  note=Cs4;   break;
      case 'd':  note=Eb4;   break;
      case 'f':  note=0;     break;
      case 'g':  note=Fs4;   break;
      case 'h':  note=Gs4;   break;
      case 'j':  note=Bb4;   break;
      case 'k':  note=0;     break;
      case 'l':  note=Cs5;   break;
      case ';':  note=Eb5;   break;
      case '\'': note=Fs5;   break;
      default: return;
   }

   if (note < 0)  note = 0;
   else if (note > 127)  note = 127;

   attack = rand()%47 + 81;           // random int from 1 to 127
   synth.play(0,note,attack); 

   noteMessage.time = mainTimer.getTime();
   noteMessage.command() = 0x90;
   noteMessage.p1() = note;
   noteMessage.p2() = rand()%47 + 81;      // random int from 1 to 127
   synth.insert(noteMessage);

}




#endif  // SYNTHIMPROV_H_INCLUDED


