//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 21 December 1997
// Last Modified: Sun Jan 25 15:45:18 GMT-0800 1998
// Filename:      ...sig/code/control/MidiIO/MidiIO.cpp
// Syntax:        C++
// $Smake:        cc -g -c %b.cpp -I../../../include && rm -f %b.o
// 
// Description: A unified object that handles MIDI input and output.
//


#include "MidiIO.h"


//////////////////////////////
//
// MidiIO::MidiIO
//

MidiIO::MidiIO(void) : MidiOutput(), MidiInput() {
   // does nothing
}


MidiIO::MidiIO(int outPort, int inPort) :
      MidiOutput(outPort), MidiInput(inPort) {
   // does nothing
}



//////////////////////////////
//
// MidiIO::~MidiIO
//

MidiIO::~MidiIO() {
   // does nothing
}



//////////////////////////////
//
// MidiIO::close
//

void MidiIO::close(void) {
   MidiInput::close();
   MidiOutput::close();
}



//////////////////////////////
//
// MidiIO::closeInput
//

void MidiIO::closeInput(void) {
   MidiInput::close();
}



//////////////////////////////
//
// MidiIO::closeOutput
//

void MidiIO::closeOutput(void) {
   MidiOutput::close();
}



//////////////////////////////
//
// MidiIO::getInputPort
//

int MidiIO::getInputPort(void) {
   return MidiInput::getPort();
}



//////////////////////////////
//
// MidiIO::getInputTrace
//

int MidiIO::getInputTrace(void) {
   return MidiInput::getTrace();
}



//////////////////////////////
//
// MidiIO::getNumInputPorts
//

int MidiIO::getNumInputPorts(void) {
   return MidiInput::getNumPorts();
}



//////////////////////////////
//
// MidiIO::getNumOutputPorts
//

int MidiIO::getNumOutputPorts(void) {
   return MidiOutput::getNumPorts();
}



//////////////////////////////
//
// MidiIO::getOutputPort
//
 
int MidiIO::getOutputPort(void) {
   return MidiOutput::getPort();
}



//////////////////////////////
//
// MidiIO::getOutputTrace
//

int MidiIO::getOutputTrace(void) {
   return MidiOutput::getTrace();
}



//////////////////////////////
//
// MidiIO::open
//

int MidiIO::open(void) {
   if (MidiInput::open()) {
      return MidiOutput::open();
   } else {
      return 0;
   }
}



//////////////////////////////
//
// MidiIO::openInput
//

int MidiIO::openInput(void) {
   return MidiInput::open();
}



//////////////////////////////
//
// MidiIO::openOutput
//

int MidiIO::openOutput(void) {
   return MidiOutput::open();
}



//////////////////////////////
//
// MidiIO::setInputPort
//   
 
void MidiIO::setInputPort(int aPort) {
   MidiInput::setPort(aPort);
}



//////////////////////////////
//
// MidiIO::setInputTrace
//

void MidiIO::setInputTrace(int aState) {
   MidiInput::setTrace(aState);
}



//////////////////////////////
//
// MidiIO::setOutputPort
//

void MidiIO::setOutputPort(int aPort) {
   MidiOutput::setPort(aPort);
}



//////////////////////////////
//
// MidiIO::setOutputTrace
//

void MidiIO::setOutputTrace(int aState) {
   MidiOutput::setTrace(aState);
}



//////////////////////////////
//
// MidiIO::toggleInputTrace
//

void MidiIO::toggleInputTrace(void) {
   MidiInput::toggleTrace();
}


//////////////////////////////
//
// MidiIO::toggleOutputTrace
//

void MidiIO::toggleOutputTrace(void) {
   MidiOutput::toggleTrace();
}



