//
// Programmer:    Craig Stuart Sapp
// Creation Date: Sat Feb 14 14:34:19 GMT-0800 1998
// Last Modified: Sat Feb 14 16:05:35 GMT-0800 1998
// Filename:      ...sig/code/control/PadPage/PadPage.cpp
// Syntax:        C++ 
// $Smake:        cc -Wall -g -c %b.cpp -I../../../include && rm -f %b.o
//
// Description: useful for parameter organization.
//

#ifndef _PADPAGE_CC_INCLUDED
#define _PADPAGE_CC_INCLUDED


#include "PadPage.h"
#include <stdlib.h>


//////////////////////////////
//
// PadPage::PadPage
//	default value: aSize = 1024
//

template<class type>
PadPage<type>::PadPage(int aSize) {
   array = NULL;
   cellUsage = NULL;
   cellFree = NULL;
   listTopFree = 0;
   arraySize = 0;

   setSize(aSize);
}



//////////////////////////////
//
// PadPage::~PadPage
//

template<class type>
PadPage<type>::~PadPage(void) {
   if (array != NULL) {
      delete [] array;
      array = NULL;
   }
   if (cellUsage != NULL) {
      delete [] cellUsage;
      array = NULL;
   }
   if (cellFree != NULL) {
      delete [] cellFree;
      array = NULL;
   }
}



//////////////////////////////
//
// PadPage::aquire -- reserves space in the page for an element.
//	Returns -1 if there is no space left to store element.
//	Use free to release the space again.
//

template<class type>
int PadPage<type>::aquire(void) {
   int returnIndex;
   if (listTopFree >= getSize()-1)  return -1;
   returnIndex = listTopFree;
   cellUsage[listTopFree] = 1;
   listTopFree++;

   return returnIndex;
}



//////////////////////////////
//
// PadPage::empty -- frees all elements in the array.
//

template<class type>
void PadPage<type>::empty(void) {
   int i;
   for (i=0; i<getSize(); i++) {
      cellUsage[i] = 0;
      cellFree[i] = i;
   }
   listTopFree = 0;
}



//////////////////////////////
//
// PadPage::free -- frees space taken up by element for reuse.
//

template<class type>
void PadPage<type>::free(int index) {
   if (index < 0) return;
   if (index >= getSize()) {
      cerr << "Error cannot access element " << index 
           << ". Maximum is: " << getSize() -1 << endl;
      exit(1);
   }

   if (cellUsage[index] == 1) {
      cellUsage[index] = 0;
      listTopFree--;
      cellFree[listTopFree] = index;
   }
}



//////////////////////////////
//
// PadPage::initialize -- sets all unused elements in the
//	storage area to the specified value.
//

template<class type>
void PadPage<type>::initialize(type& aThing) {
   int i;
   for (i=0; i<getSize(); i++) {
      if (cellUsage[i] == 0) {
         array[i] = aThing;
      }
   }
}



//////////////////////////////
//
// PadPage::insert -- reserves space and copyies aThing into the
//	page.  Returns the index of the element storage area in the
//	page, or -1 if no space is available to insert element.
//

template<class type>
int PadPage<type>::insert(type& aThing) {
   int returnIndex;
   if (listTopFree >= getSize())  return -1;
   returnIndex = cellFree[listTopFree];
   listTopFree++;
   array[returnIndex] = aThing;
   cellUsage[returnIndex] = 1;
   return returnIndex;
}



//////////////////////////////
//
// PadPage::getNumFreeSpaces -- returns the number of elements 
//	which are free and can be reserved in the page.
//

template<class type>
int PadPage<type>::getNumFreeSpaces(void) const {
   return getSize() - listTopFree;
}



//////////////////////////////
//
// PadPage::getNumUsedSpaces -- returns the number of elements 
//	being used in the page.
//

template<class type>
int PadPage<type>::getNumUsedSpaces(void) const {
   return listTopFree;
}



//////////////////////////////
//
// PadPage::getSize -- returns the maximum number of elements
//	which can be inserted into the page.
//

template<class type>
int PadPage<type>::getSize(void) const {
   return arraySize;
}



//////////////////////////////
//
// PadPage::getStatus -- returns true if the element index is being
//	used.  Returns fase if the element index is not reserved.
//

template<class type>
int PadPage<type>::getStatus(int index) {
   if (index < 0 || index >= getSize()) {
      cerr << "Error trying to access an invalid element: " << index
           << ".  Range is 0 to " << getSize()-1 << endl;
      exit(1);
   }

   return (int)cellUsage[index];
}



//////////////////////////////
//
// PadPage::operator[] -- treat page as an array.
//

template<class type>
type& PadPage<type>::operator[](int index) {
   // assume that index is valid.
   return array[index];
}



//////////////////////////////
//
// PadPage::setSize -- frees all spaces in page and uninitializes
//	any initialized spaces.
//

template<class type>
void PadPage<type>::setSize(int aSize) {
   if (aSize < 0) return;

   if (array != NULL) {
      delete [] array;
      array = NULL;
   }
   if (cellUsage != NULL) {
      delete [] cellUsage;
      array = NULL;
   }
   if (cellFree != NULL) {
      delete [] cellFree;
      array = NULL;
   }

   arraySize = aSize;
   array = new type[aSize];
   cellUsage = new char[aSize];
   cellFree =  new int[aSize];

   int i;
   for (i=0; i<aSize; i++) {
      cellUsage[i] = 0;
      cellFree[i] = i;
   }
   listTopFree = 0;
}



#endif  // _PADPAGE_CC_INCLUDED


